@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: command.c,v 1.126.2.2 2002/12/03 21:33:17 priikone Exp $ */
+/* $Id: command.c,v 1.126.2.3 2003/02/01 10:43:37 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcclient.h"
@@ -1175,6 +1175,8 @@ SILC_CLIENT_CMD_FUNC(join)
   silc_buffer_free(idp);
   if (auth)
     silc_buffer_free(auth);
+  if (passphrase)
+    memset(passphrase, 0, strlen(passphrase));
   silc_free(passphrase);
 
   /* Notify application */
@@ -1935,6 +1937,7 @@ static void silc_client_command_oper_send(unsigned char *data,
 			  0, NULL, NULL, buffer->data, buffer->len, TRUE);
 
   silc_buffer_free(buffer);
+  silc_buffer_clear(auth);
   silc_buffer_free(auth);
 
   /* Notify application */
@@ -2006,6 +2009,7 @@ static void silc_client_command_silcoper_send(unsigned char *data,
 			  0, NULL, NULL, buffer->data, buffer->len, TRUE);
 
   silc_buffer_free(buffer);
+  silc_buffer_clear(auth);
   silc_buffer_free(auth);
 
   /* Notify application */
